/*
	$Id$

	Copyright (C) 2024 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import net.sf.basedb.clients.web.Base;
import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.AnnotationTypeCategory;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Subtypable;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restrictions;

/**
	This tag defines a selectable list of subtypes.

	@author Nicklas
	@since 3.20
*/
public class SubtypeSelector
	extends StylableTag
{

	private static final long serialVersionUID = 4572474689501409001L;
	
	private Subtypable item;
	public void setItem(Subtypable item)
	{
		this.item = item;
	}
	public Subtypable getItem()
	{
		return item;
	}
	
	private Item itemType;
	public void setItemtype(Item itemType)
	{
		this.itemType = itemType;
	}
	public Item getItemtype()
	{
		return itemType;
	}
	
	private DbControl dc;
	public void setDc(DbControl dc)
	{
		this.dc = dc;
	}
	public DbControl getDc()
	{
		return dc;
	}
	
	private String table;
	public void setTable(String table)
	{
		this.table = table;
	}
	public String getTable()
	{
		return table;
	}
	
	private ItemSubtype current;
	public void setCurrent(ItemSubtype current)
	{
		this.current = current;
	}
	public ItemSubtype getCurrent()
	{
		return current;
	}
	private AnnotationTypeCategory currentSubCategory;
	public void setSubcategory(AnnotationTypeCategory category)
	{
		this.currentSubCategory = category;
	}
	public AnnotationTypeCategory getSubcategory()
	{
		return currentSubCategory;
	}
	
	private List<ItemSubtype> subtypes;
	public void setSubtypes(List<ItemSubtype> subtypes)
	{
		this.subtypes = subtypes;
	}	
	public List<ItemSubtype> getSubtypes()
	{
		return subtypes;
	}
	
	public SubtypeSelector()
	{
		super("subtypeselector");
	}
	
	@Override
	public int doStartTag() 
		throws JspException
	{
		// Need local variable since taglibs are cached and not thread-safe
		Item itemType = getItemtype();
		List<ItemSubtype> subtypes = getSubtypes();
		ItemSubtype current = getCurrent();
		AnnotationTypeCategory currentSubCategory = getSubcategory();
		DbControl dc = getDc();
		
		if (item != null)
		{
			try
			{
				if (itemType == null) itemType = item.getType();
				if (current == null) current = item.getItemSubtype();
				if (dc == null) dc = ((BasicItem)item).getDbControl();
				if (currentSubCategory == null && current != null && item instanceof Annotatable)
				{
					// Try to match the item against an annotation type category
					Annotatable aItem = (Annotatable)item;
					AnnotationType subCatType = current.getSubCategoriesAnnotationType();
					if (subCatType != null && aItem.isAnnotated() && aItem.getAnnotationSet().hasAnnotation(subCatType, null))
					{
						List<?> values = aItem.getAnnotationSet().getAnnotation(subCatType).getValues();
						ItemQuery<AnnotationTypeCategory> query = current.getCategories(values);
						query.setIncludes(Include.ALL);
						List<AnnotationTypeCategory> categories = query.list(dc);
						if (categories.size() >= 1)
						{
							currentSubCategory = categories.get(0);
						}
					}
				}
			}
			catch (PermissionDeniedException ex)
			{} // Ignore this and continue with what we have
		}
		if (subtypes == null && dc != null)
		{
			subtypes = Base.getSubtypesQuery(itemType).list(dc);
		}
		if (subtypes == null || subtypes.isEmpty()) return SKIP_BODY;
		
		ItemQuery<AnnotationTypeCategory> subCategoryQuery = AnnotationTypeCategory.getQuery();
		subCategoryQuery.restrict(Restrictions.eq(Hql.property("itemSubtype"), Hql.entityParameter("subtype", Item.ITEMSUBTYPE)));
		subCategoryQuery.order(Orders.asc(Hql.property("name")));
		subCategoryQuery.setCacheResult(true);
		
		JSONArray jsonSubtypes = new JSONArray();
		JSONObject jsonCurrent = new JSONObject();
		for (ItemSubtype st : subtypes)
		{
			if (dc == null) dc = st.getDbControl();
			JSONObject jsonSt = new JSONObject();
			jsonSt.put("id", st.getId());
			jsonSt.put("name", st.getName());
			jsonSubtypes.add(jsonSt);
			
			if (st.equals(current))
			{
				jsonCurrent.put("name", st.getName());
				jsonCurrent.put("subtype", jsonSt);
			}
			
			subCategoryQuery.setEntityParameter("subtype", st);
			List<AnnotationTypeCategory> subCategories = subCategoryQuery.list(dc);
			int numSubCategories = subCategories.size();
			if (numSubCategories > 0)
			{
				JSONArray jsonSubCategories = new JSONArray();
				for (AnnotationTypeCategory cat : subCategories)
				{
					JSONObject jsonCat = new JSONObject();
					jsonCat.put("id", cat.getId());
					jsonCat.put("name", cat.getName());
					jsonCat.put("shortName", cat.getShortName());
					jsonSubCategories.add(jsonCat);
					
					if (cat.equals(currentSubCategory))
					{
						jsonCurrent.put("name", cat.getName());
						jsonCurrent.put("subCategory", jsonCat);
					}
				}
				if (jsonSubCategories.size() > 0)
				{
					jsonSt.put("subCategories", jsonSubCategories);
				}
			}
		}
		
		StringBuilder sb = new StringBuilder();
		sb.append("<span");
		addIdAndStyles(sb, "auto-init");
		addDynamicAttributes(sb);
		sb.append(" data-auto-init=\"subtype-selector\"");
		sb.append(" data-item-type=\""+itemType.name()+"\"");
		if (table != null) sb.append(" data-table=\""+table+"\"");
		sb.append(" data-subtypes=\""+HTML.encodeTags(jsonSubtypes.toJSONString())+"\"");
		if (jsonCurrent.size() > 0)
		{
			sb.append(" data-current=\""+HTML.encodeTags(jsonCurrent.toJSONString())+"\"");
		}
		sb.append(" title=\"Select another subtype\"");
		sb.append(">");
		sb.append("<div class=\"current-subtype\">");
		if (currentSubCategory != null)
		{
			sb.append(HTML.encodeTags(currentSubCategory.getName()));
		}
		else if (current != null)
		{
			sb.append(HTML.encodeTags(current.getName()));
		}
		else
		{
			sb.append("<i>All types</i>");
		}
		sb.append("</div>");
		sb.append("</span>");
		try
		{
			pageContext.getOut().print(sb.toString());
    	}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return SKIP_BODY;
	}
}


