/*
  $Id: AnnotationTypeCategory.java 8263 2024-10-18 11:41:27Z nicklas $

  	Copyright (C) 2006 Nicklas Nordborg, Dominic Oyeniran, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.core;

import java.util.Collections;
import java.util.Date;
import java.util.List;

import net.sf.basedb.core.data.AnnotationTypeCategoryData;
import net.sf.basedb.core.data.AnnotationTypeData;
import net.sf.basedb.core.data.ItemSubtypeData;
import net.sf.basedb.core.data.ParameterValueData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;


/**
	Objects of this class defines an annotation type category. 
	The categories are used to group related {@link AnnotationType}:s.
	<p>
 	An annotation type can belong to more than one category. 
 	
 	@author Dominic Oyeniran
	@version 2.0
	@see AnnotationType
	@base.modified $Date: 2006-05-11 (Thu, 11 May 2006) $
*/

public class AnnotationTypeCategory 
	extends CommonItem
	implements Registered

{
	/**
	 	The type of item represented by this class.
	 	@see Item#ANNOTATIONTYPECATEGORY
	 	@see #getType()
    */
	public static final Item TYPE = Item.ANNOTATIONTYPECATEGORY;
	
		
	/**
	 	Creates a new <code>AnnotationTypeCategory</code> item.
	 	@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@return The new <code>AnnotationType</code> item
		@throws BaseException If there is an error
	*/
	public static AnnotationTypeCategory getNew(DbControl dc)
		throws BaseException
	{
		AnnotationTypeCategory atcg = dc.newItem(AnnotationTypeCategory.class);
		atcg.setName("New category");
		atcg.getData().setEntryDate(new Date());
		return atcg;
	}

	/**
	 	Get an <code>AnnotationTypeCategory</code> item when you know the id.
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param id The id of the item to load
		@return The <code>AnnotationTypeCategory</code> item
		@throws ItemNotFoundException If an item with the specified id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static AnnotationTypeCategory getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		AnnotationTypeCategory atcg = dc.loadItem(AnnotationTypeCategory.class, id);
		if (atcg == null) throw new ItemNotFoundException("AnnotationTypeCategory[id="+id+"]");
		return atcg;
	}

	/**
		Get a {@link ItemQuery} object configured to retrieve <code>AnnotationTypeCategory</code> 
		items.
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<AnnotationTypeCategory> getQuery()
	{
		return new ItemQuery<AnnotationTypeCategory>(AnnotationTypeCategory.class);
	}
	

	AnnotationTypeCategory(AnnotationTypeCategoryData annotationTypeCategoryData) 
	{
		super(annotationTypeCategoryData);
	}
	
	@Override
	AnnotationTypeCategoryData getData()
	{
		return (AnnotationTypeCategoryData)getBasicData();
	}
	
	/*
	 	From the Identifiable interface
	 	-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	@Override
	public void setEntryDate(Date entryDate)
	{
		if (isInDatabase()) 
		{
			throw new PermissionDeniedException("Not allowed to change entryDate of " + this);
		}
		getData().setEntryDate(DateUtil.setDateOrToday(entryDate));
	}
	// -------------------------------------------

	/**
		A short name can be used in some places instead of the normal name.
		@since 3.20
	*/
	public String getShortName()
	{
		return getData().getShortName();
	}
	/**
		Set the short name.
		@since 3.20
	*/
	public void setShortName(String shortName)
	{
		checkPermission(Permission.WRITE);
		getData().setShortName(StringUtil.setNullableString(shortName, "shortName", Nameable.MAX_NAME_LENGTH));
	}
	
	/**
		Get the item subtype that is linked to this category of annotations.
		@since 3.20
	*/
	public ItemSubtype getItemSubtype()
	{
		return getDbControl().getItem(ItemSubtype.class, getData().getItemSubtype());
	}
	/**
		Set the item subtype that is linked to this category of annotations.
		@since 3.20
	*/
	public void setItemSubtype(ItemSubtype subtype)
	{
		checkPermission(Permission.WRITE);
		getData().setItemSubtype(subtype == null ? null : subtype.getData());
	}
	
	/**
		Short-cut for calling getItemSubtype().getSubCategoriesAnnotationType().
		@since 3.20
	*/
	public AnnotationType getSubCategoriesAnnotationType()
	{
		ItemSubtypeData subtype = getData().getItemSubtype();
		return subtype == null ? null : getDbControl().getItem(AnnotationType.class, subtype.getSubCategoriesAnnotationType());
	}
	
	/**
		Short-cut for calling getSubCategoriesAnnotationType().getValueType()
		@since 3.20
	*/
	public Type getSubCategoriesValueType()
	{
		ItemSubtypeData subtype = getData().getItemSubtype();
		AnnotationTypeData at = subtype == null ? null : subtype.getSubCategoriesAnnotationType();
		return at == null ? null : Type.fromValue(at.getValueType());
	}
	
	/**
		Short-cut for calling getSubCategoriesAnnotationType().getValues()
		@since 3.20
	*/
	public <T> List<T> getSubCategoriesEnumeration()
	{
		List<T> values = null;
		ItemSubtypeData subtype = getData().getItemSubtype();
		if (subtype != null)
		{
			AnnotationTypeData at = subtype.getSubCategoriesAnnotationType();
			if (at != null)
			{
				ParameterValueData<?> pv = at.getEnumerationValues();
				if (pv != null)
				{
					values = Values.getItemValues(getDbControl(), pv.getValues());
					if (values.size() > 1) values.sort(null);
				}
			}
		}
		return values == null || values.isEmpty() ? null : Collections.unmodifiableList(values);
	}
	
	/**
		Get the list of values that are used to match this category.
		@since 3.20
	*/
	public <T> List<T> getSubCategoryValues()
		throws BaseException
	{
		ParameterValueData<?> pv = getData().getSubCategoryValues();
		if (pv == null) return null;
		
		List<T> values = Values.getItemValues(getDbControl(), pv.getValues());
		if (values.size() > 1) values.sort(null);
		return Collections.unmodifiableList(values);
	}
	
	/**
		Set the list of values that should be used to match sub-categories.
		The list of values must be taken from the annotation type that
		is used as the sub-category annotation type on the associated
		item subtype.
		
		@see ItemSubtype#getSubCategoriesAnnotationType()
		@since 3.20
	*/
	public void setSubCategoryValues(List<?> values)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		
		if (values != null && !values.isEmpty())
		{
			ItemSubtypeData subtype = getData().getItemSubtype();
			if (subtype == null)
			{
				throw new InvalidDataException("This category must have a linked item subtype: "+getName());
			}
			AnnotationTypeData at = subtype.getSubCategoriesAnnotationType();
			if (at == null)
			{
				throw new InvalidDataException("The linked item subtype must specify an annotation type: "+subtype.getName());
			}
			Type valueType = Type.fromValue(at.getValueType());
			valueType.validate(values);
			ParameterValueData<?> pv = getData().getSubCategoryValues();
			if (pv == null)
			{
				pv = valueType.newParameterValueData();
				getData().setSubCategoryValues(pv);
			}
			
			// TODO -- verify that values are found in the annotation type
			
			pv.replaceValuesIfDifferent(Values.getDataValues(values), false);
		}
		else
		{
			getData().setSubCategoryValues(null);
		}
	}

	
	/** 
		Gets a query that returns the annotation types that are members of this category
		@param itemType Optional, limit the annotation types to those
			that are enabled for the specified item
		@return An <code>ItemQuery</code> object
	*/
	public ItemQuery<AnnotationType> getAnnotationTypes(Item itemType)
	{
		ItemQuery<AnnotationType> query= AnnotationType.getQuery(itemType);
		query.joinPermanent(Hql.innerJoin("categories", Item.ANNOTATIONTYPECATEGORY.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(Item.ANNOTATIONTYPECATEGORY.getAlias()), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Checks if the given annotation type is a member in this category.
		@since 3.20
	*/
	public boolean isInCategory(AnnotationType at)
	{
		return getData().getAnnotationTypes().contains(at.getData());
	}
}
